"use strict";

var fs = require("fs");
var util = require("util");
var common = require("./common");
var EventEmitter = require("events").EventEmitter;

var ejs = require("ejs");
//Mu.root = "./templates";


function CachingClient(bridge) {
    this.bridge = bridge
}

var queries = {
    "__all all AND itemtype=*&groupby=provider,decade,language,itemtype": "rs-c1"
};
var rSets = {
    "rs-c1": {
        params: {
            index: "all",
            query: "__all all AND itemtype=*&groupby=provider,decade,language,itemtype",
            mdformat: "oai_efg",
            layout: "index",
        },
        method: "browse",
        fromPosition: 1,
        toPosition: 4
    },
    "rs-c2": {
        params: {
            id: "all",
            query: "*",
            mdformat: "oai_efg",
            layout: "index"
        },
        method: "index",
        fromPosition: 1,
        toPosition: 4
    }

};

var nElCache = {};

var resultCache = {};



var eprTemplate;

fs.readFile("epr.xml", "utf-8", function(err, str) {
    if (err) throw err;
    eprTemplate = ejs.compile(str);
});

/* if res == null, behaves like a sink */
function ResInterceptor(res) {
    this.res = res;
};

util.inherits(ResInterceptor, EventEmitter);

ResInterceptor.prototype.writeHead = function(status, data) {
    if(this.res)
        return this.res.writeHead(status, data);
};

ResInterceptor.prototype.end = function(data) {
//    console.log("intercepting", data);
    this.emit("data", data);
    if(this.res)
        return this.res.end(data);
};

function parseEpr(data) {
    var addr, rsId;

    var match = data.match(/Address>([^<])*/);
    if(match)
        addr = match[0].substr(8);

    match = data.match(/>rs-([^<])*/);
    if(match)
        rsId = match[0].substr(1);

    return {addr: addr, rsId: rsId};
}


CachingClient.prototype = {
    init: function() {
        console.log("caching enabled");
        this.bridge.init();
    },
    
    indexLookup: function(params, res) {
//        console.log("caching");
        return this.bridge.indexLookup(params, res);
    },

    getBrowsingStatistics: function(params, res) {
//        console.log("caching getBrowsingStatistics");
        var query = params.query;
        var rs = queries[query];
        if(rs) {
            console.log("returning caching rs");
            var epr = eprTemplate({rsid: rs});
            common.reply(res)(null, epr);
        } else {
            return this.bridge.getBrowsingStatistics(params, res);
        }
    },

    getNumberOfElements: function(params, res) {
//        console.log("caching getNumberOfElements");
        var self = this;
        this.fillCache(params, res, nElCache, function(params, epr, res) {
            self.bridge.getNumberOfElements(epr, res);
        });
    },

    getResult: function(params, res) {
//        console.log("caching getResult");
        var self = this;
        this.fillCache(params, res, resultCache, function(params, epr, res) {
            self.bridge.getResult({rsId: epr.rsId, fromPosition: params.fromPosition, toPosition: params.toPosition}, res);
        }, function (p, up, cp) {
            return p.fromPosition == cp.fromPosition && p.toPosition == cp.toPosition;
        });
    },

    stats: function(params, res, next) {
//        console.log("caching");
        res.end("callback queue length: " + callbacks.length);
    },
    
    fillCache: function(params, res, cache, upstream, verify) {
        var rsId = params.rsId;
        var upstreamParams = rSets[rsId];

        var cached = cache[rsId];
        if(verify === undefined)
            verify = function() { return true; };
        if(cached && verify(params, upstreamParams, cached.params)) {
//            console.log("cache HIT");
            common.reply(res)(null, cached.data);
        } else {            
            var epr = new ResInterceptor();
            var self = this;
            epr.on("data", function(data) {
                data = "" + data;
                var epr = parseEpr(data);
                
                var elements = new ResInterceptor();
                elements.on("data", function(data) {
                    cache[rsId] = {data: ""+data, params: params};
                    
                    common.reply(res)(null, data);
                });
                upstream(params, epr, elements);
            });
            
//            console.log("cache MISS, asking upstream");
            if(upstreamParams.method == "browse")
                this.bridge.getBrowsingStatistics(upstreamParams.params, epr);
            else
                this.bridge.indexLookup(upstreamParams.params, epr);
        }
                
    }
};

exports.Client = CachingClient;