#!/usr/bin/env node

"use strict";

var connect = require("connect");
var http = require("http");
var common = require("./common");
var params = common.params;

var argv = require('optimist').boolean(['dummy', 'thrift', 'zeromq', 's', 'caching']).usage('Usage: $0 [--thrift|--zeromq]').argv;

//argv.dummy = true;

// Constructor four our bridge object.
function HttpBridge(client) {
    this.client = client;
}

HttpBridge.prototype = {    
    indexLookup: function(req, res, next) {
        this.client.indexLookup(params(req), res);
    },

    getBrowsingStatistics: function(req, res, next) {
        this.client.getBrowsingStatistics(params(req), res);
    },

    getNumberOfElements: function(req, res, next) {
        this.client.getNumberOfElements(params(req), res);
    },

    getResult:  function(req, res, next) {
        this.client.getResult(params(req), res);
    },

    stats: function(req, res, next) {
        this.client.stats(req, res, next);
    }
};


module.exports = connect(
    connect.router(function(app) {

        var bridge;

        if(argv.thrift) {
            console.log("using thrift");
            var thrift = require("./thrift");
            bridge = new thrift.Client();
        } else if(argv.dummy) {
            console.log("using dummy");
            var dummy = require("./dummy");
            bridge = new dummy.Client();
        } else {
            console.log("using zeromq");
            var zmq = require("./zmq");
            bridge = new zmq.Client(argv.s);
        }

        if(argv.caching) {
            var caching = require("./caching");
            bridge = new caching.Client(bridge);
        }

        bridge.init();

        var client = new HttpBridge(bridge);

        app.get('/stats', client.stats.bind(client));

        app.get('/IndexService/indexLookup',  client.indexLookup.bind(client));
        app.get('/IndexService/getBrowsingStatistics',  client.getBrowsingStatistics.bind(client));
        app.get('/resultSet/getNumberOfElements', client.getNumberOfElements.bind(client));
        app.get('/resultSet/getResult', client.getResult.bind(client));

        app.get('/:context/services/IndexService/indexLookup',  client.indexLookup.bind(client));
        app.get('/:context/services/IndexService/getBrowsingStatistics',  client.getBrowsingStatistics.bind(client));
        app.get('/:context/services/resultSet/getNumberOfElements', client.getNumberOfElements.bind(client));
        app.get('/:context/services/resultSet/getResult', client.getResult.bind(client));

        app.get('/:context/mvc/directindex.do/IndexService/indexLookup',  client.indexLookup.bind(client));
        app.get('/:context/mvc/directindex.do/IndexService/getBrowsingStatistics',  client.getBrowsingStatistics.bind(client));
        app.get('/:context/mvc/directindex.do/resultSet/getNumberOfElements', client.getNumberOfElements.bind(client));
        app.get('/:context/mvc/directindex.do/resultSet/getResult', client.getResult.bind(client));

    })
); // .listen(3000);
